@@ -17,6 +17,9 @@ class EnumValue(object):
 
 
 class Enum(object):
+    class NotSet:
+        pass
+
     def __init__(self, **kwargs):
         self._values = []
         for key, value in list(kwargs.items()):
@@ -26,17 +29,18 @@ class Enum(object):
 
     def find(self,
              identifier):
-        for  value in self._values:
+        for value in self._values:
             if identifier == value.identifier:
                 return value
-        return None
+        return Enum.NotSet()
 
     def from_id(self,
                 identifier):
-        if self.find(identifier) is None:
+        found = self.find(identifier)
+        if isinstance(found, Enum.NotSet):
             raise ValueError('Unexpected identifier {0}'.format(identifier))
         else:
-            return self.find(identifier)
+            return found
 
     def __contains__(self, value):
         return value in self._values
@@ -96,8 +100,7 @@ FieldKind = Enum(regular = 'regular',
 
 DocumentStatus = Enum(published = 'PUBLISHED',
                       new = 'NEW',
-                      approved = 'APPROVED',
-                      unapproved =  'UNAPPROVED',
+                      unpublished = 'UNPUBLISHED',
                       deleted = 'DELETED')
 
 
@@ -154,42 +157,48 @@ AcademicCategory = Enum(bachelor_thesis = 1,
                         submitted_assignment = 49,
                         presentation_slides = 50)
 
-Language = ExtendableEnum(German = 'ger',
-                          Danish = 'dan',
-                          English = 'eng',
-                          French = 'fre',
-                          Dutch = 'dut',
-                          Italian = 'ita',
-                          Catalan = 'cat',
-                          Croatian = 'scr',
-                          Latin = 'lat',
-                          Norwegian = 'nor',
-                          Polish = 'pol',
-                          Portugues = 'por',
-                          Russian = 'rus',
-                          Swedish = 'swe',
-                          Czech = 'cze',
-                          Serbian = 'scc',
-                          Greek = 'gre',
-                          Spanish = 'spa',
-                          Estonian = 'est',
-                          Turkish = 'tur',
-                          Finnish = 'fin',
-                          Hungarian = 'hun',
-                          Romanian = 'rum',
-                          Arabic = 'ara',
-                          Mandarin = 'chi',
-                          Icelandic = 'ice',
-                          Albanian = 'alb',
-                          Lithuanian = 'lit',
-                          Afrikaans = 'afr',
-                          Slovakian = 'slo',
-                          Korean = 'kor',
-                          Slovenian = 'slv',
-                          Thai = 'tha',
-                          Urdu = 'urd',
-                          Tsonga = 'tso',
-                          Bosnian = 'bos')
+
+Language = ExtendableEnum(German = 'de',
+                          Danish = 'da',
+                          English = 'en',
+                          French = 'fr',
+                          Dutch = 'nl',
+                          Italian = 'it',
+                          Catalan = 'ca',
+                          Croatian = 'hr',
+                          Latin = 'la',
+                          Norwegian = 'no',
+                          Polish = 'pl',
+                          Portugues = 'pt',
+                          Russian = 'ru',
+                          Swedish = 'sv',
+                          Serbian = 'sr',
+                          Spanish = 'es',
+                          Turkish = 'tr',
+                          Hungarian = 'hu',
+                          Arabic = 'ar',
+                          Czech = 'cs',
+                          Greek = 'el',
+                          Estonian = 'et',
+                          Finnish = 'fi',
+                          Romanian = 'ro',
+                          Icelandic = 'is',
+                          Lithuanian = 'lt',
+                          Slovakian = 'sk',
+                          Slovenian = 'sl',
+                          Urdu = 'ur',
+                          Mandarin = 'zh',
+                          Albanian = 'sq',
+                          Afrikaans = 'af',
+                          Korean = 'ko',
+                          Thai = 'th',
+                          Tsonga = 'ts',
+                          Bosnian = 'bs',
+                          Bulgarian = 'bg',
+                          Ukrainian = 'uk',
+                          Vietnamese = 'vi',
+                          Hebrew = 'he',
+                          Japanese = 'ja')
 
 
 Country = ExtendableEnum(Afghanistan = 'AFG',
@@ -407,7 +416,10 @@ ThemaCode = ExtendableEnum()
 FileType = Enum(ebook_pdf = 'ebook_pdf',
                 ebook_epub = 'ebook_epub',
                 ebook_mobi = 'ebook_mobi',
+                ebook_ibooks = 'ebook_ibooks',
                 pod_pdf = 'pod_pdf',
+                audiobook = 'audiobook',
+                software = 'software',
                 cover_pod_pdf_final = 'cover_pod_pdf_final',
                 cover_marketing_jpg = 'cover_marketing_jpg',
                 other = 'other')
@@ -423,18 +435,19 @@ CatalogType = Enum(academic = 'academic',
 
 EBookFileType = Enum(pdf = 'pdf',
                      epub = 'epub',
-                     mobi = 'mobi')
+                     mobi = 'mobi',
+                     ibooks = 'ibooks')
 
 PreviewFileType = Enum(pdf= 'pdf',
                     html = 'html')
 
-ContentType = Enum(final = 'final',
-                   production = 'production')
-
 IsbnType = Enum(book = 'book',
                 epub = 'ebook::epub',
                 mobi = 'ebook::mobi',
+                ibooks = 'ebook::ibooks',
                 pdf = 'ebook::pdf',
+                audiobook = 'audiobook',
+                software = 'software',
                 ebook = 'ebook')
 
 EventTarget = Enum(book = 'book',
@@ -443,6 +456,9 @@ EventTarget = Enum(book = 'book',
                    ebook = 'ebook',
                    epub = 'epub',
                    mobi = 'mobi',
+                   ibooks = 'ibooks',
+                   audiobook = 'audiobook',
+                   software = 'software',
                    raw_document = 'raw-document',
                    vlb_onix = 'vlb-onix',
                    preview = 'preview',
@@ -457,12 +473,11 @@ EventAction = Enum(price_changed = 'price-changed',
                    created = 'created',
                    uploaded = 'uploaded',
                    changed = 'changed',
-                   approved = 'approved',
                    blocked = 'blocked',
                    deleted = 'deleted',
                    publication_availability_changed = 'publication-availability-changed',
                    published = 'published',
-                   unapproved = 'unapproved',
+                   unpublished = 'unpublished',
                    creation = 'creation',
                    subscribed = 'subscribed',
                    unsubscribed = 'unsubscribed',
@@ -499,7 +514,8 @@ UserStatus = Enum(active = 'ACTIVE',
 
 Gender = Enum(male = 'M',
               female = 'F',
-              unknown = 'NULL')
+              diverse = 'D',
+              unknown = None)
 
 Subscription = Enum(author = 'author',
                     buyer = 'buyer',
@@ -508,6 +524,9 @@ Subscription = Enum(author = 'author',
 PublicationType = Enum(pdf = 'pdf',
                        epub = 'epub',
                        mobi = 'mobi',
+                       ibooks = 'ibooks',
+                       audiobook = 'audiobook',
+                       software = 'software',
                        pod = 'pod')
 
 TaxType = Enum(vat_percentage = 'vat_percentage',
@@ -517,9 +536,34 @@ TaxType = Enum(vat_percentage = 'vat_percentage',
 OnixStyle = Enum(amazon = 'amazon',
                  google = 'google',
                  vlb = 'vlb',
-                 tolino = 'tolino',
+                 overdrive = 'overdrive',
+                 skoobe = 'skoobe',
                  knv = 'knv',
                  international = 'international',
+                 ceebo = 'ceebo',
+                 ciando = 'ciando',
+                 beam = 'beam',
+                 libri = 'libri',
+                 schweitzer = 'schweitzer',
+                 rdw = 'rdw',
+                 olf = 'olf',
+                 audible = 'audible',
+                 bicmedia = 'bicmedia',
+                 bookwire = 'bookwire',
+                 divibib = 'divibib',
+                 doccheck = 'doccheck',
+                 genios = 'genios',
+                 kobo = 'kobo',
+                 lernando = 'lernando',
+                 nexway = 'nexway',
+                 parkteam = 'parkteam',
+                 pubdatanet = 'pubdatanet',
+                 softdist = 'softdist',
+                 steinsche = 'steinsche',
+                 tigerbooks = 'tigerbooks',
+                 mojoreads = 'mojoreads',
+                 amzprint = 'amzprint',
+                 wlb = 'wlb',
                  default = 'default')
 
 OnixType = Enum(short = 'short',
@@ -528,24 +572,11 @@ OnixType = Enum(short = 'short',
 
 OnixStatus = Enum(current = 'current',
                   available = 'available',
-                  announced = 'announced',
                   unavailable = 'unavailable',
                   deleted = 'deleted',
                   blocked = 'blocked')
 
 
-AdultFlag = Enum(unrated = 'UNRATED',
-                 any_adult_audience = 'ANY_ADULT_AUDIENCE',
-                 content_warning = 'CONTENT_WARNING',
-                 content_warning_sex = 'CONTENT_WARNING_SEX',
-                 content_warning_violence = 'CONTENT_WARNING_VIOLENCE',
-                 content_warning_drug_taking = 'CONTENT_WARNING_DRUG_TAKING',
-                 content_warning_language = 'CONTENT_WARNING_LANGUAGE',
-                 content_warning_intolerance = 'CONTENT_WARNING_INTOLERANCE')
-
-ChildrenFlag = Enum(unrated = 'UNRATED',
-                    children_juvenile = 'CHILDREN_JUVENILE')
-
 ProvisionRuleAlgorithm = Enum(percentage = 'PERCENTAGE',
                               fixed = 'FIXED',
                               progression = 'PROGRESSION',
@@ -566,7 +597,7 @@ ProvisionChannelType = Enum(book_and_ebook = '',
 
 
 ProvisionChannelBase = Enum(net_price = 'NET_PRICE',
-                            groos_price = 'GROSS_PRICE',
+                            gross_price = 'GROSS_PRICE',
                             net_margin = 'NET_MARGIN',
                             gross_margin = 'GROSS_MARGIN')
 
@@ -748,12 +779,20 @@ ProfileShow = Enum(no = 'NO',
                    yes = 'YES',
                    default = 'DEFAULT')
 
-AssetsCoverType = Enum(distribution = 'distribution')
+AssetsCoverType = Enum(distribution = 'distribution',
+                       original = 'original',
+                       big = 'big')
 
 AssetsModules = Enum(cover = 'cover',
                      epub = 'epub',
                      mobi = 'mobi',
+                     ibooks = 'ibooks',
+                     audiobook = 'audiobook',
+                     software = 'software',
                      pdf = 'pdf',
+                     pod = 'pod',  # Deprecated
+                     pod_bookblock = 'pod_bookblock',
+                     pod_cover = 'pod_cover',
                      extract = 'extract')
 
 BookBinding = Enum(auto = 'AUTO',
