@@ -17,6 +17,7 @@ from .assets import AssetsGroup
 from .isbns import IsbnGroup
 from .publications import PublicationsField
 from .dates import DatesGroup
+from .audio import AudioGroup
 from .audience import AudienceGroup
 from .extendable_enum_field import ExtendableEnumField
 from .object_events import ObjectEvents
@@ -25,7 +26,9 @@ from .archive import ArchiveField
 from .distribution_availability import DistributionAvailabilityField
 from .distribution_channels import DistributionChannelsField
 from .search_tags import SearchTagsField
-from .admin_search_tags import AdminSearchTagsField
+from .labels import LabelsField
+from .external_shop.shoplink import ShopLink
+from .external_shop.external_id import ExternalID
 
 class Document(DatabaseObject):
     _object_class = "document"
@@ -69,29 +72,31 @@ class Document(DatabaseObject):
                                                       field_locator="internal.self_publishing",
                                                       dtype=bool)
 
-        self._fields['language'] = ExtendableEnumField(database_object=self,
-                                                       aspect=":basic",
-                                                       field_locator="language_id",
-                                                       dtype=Language,
-                                                       nullable=True)
+        self._fields["internal_note"] = SimpleField(database_object=self,
+                                            aspect="internal.*",
+                                            field_locator="internal.note",
+                                            dtype=str)
+
+        self._fields["insert_bastard_title"] = SimpleField(database_object=self,
+                                            aspect="document_print_configuration.insert_bastard_title",
+                                            field_locator="document_print_configuration.insert_bastard_title",
+                                            dtype=bool)
+
+        self._fields["bastardtitle_additional_legal_information"] = SimpleField(database_object=self,
+                                            aspect="document_print_configuration.bastardtitle_additional_legal_information",
+                                            field_locator="document_print_configuration.bastardtitle_additional_legal_information",
+                                            dtype=str)
+
+        self._fields["language"] = SimpleField(database_object=self,
+                                            aspect="language",
+                                            field_locator="language",
+                                            dtype=str,
+                                            nullable=True)
 
         self._fields["drm"] = SimpleField(database_object=self,
                                           aspect="internal.*",
                                           field_locator="internal.drm",
                                           dtype=DRM)
-        
-        self._fields["_page_count_technical"] = SimpleField(database_object=self,
-                                                            aspect="ebook.*",
-                                                            field_locator="ebook.page_count_technical",
-                                                            dtype=int,
-                                                            nullable=True,
-                                                            kind=FieldKind.readonly)
-
-        self._fields["_page_count_custom"] = SimpleField(database_object=self,
-                                                         aspect="ebook.*",
-                                                         field_locator="ebook.page_count_custom",
-                                                         dtype=int,
-                                                         nullable=True)
 
         self._fields["urls"] = URLsGroup(document=self)
 
@@ -106,11 +111,6 @@ class Document(DatabaseObject):
                                                         field_locator="othertext_biographical_note.text",
                                                         dtype=str)
 
-        self._fields["suppliers"] = SimpleField(database_object=self,
-                                                aspect="book.suppliers_in_stock",
-                                                field_locator="book.suppliers_in_stock",
-                                                dtype=set)
-
         
         self._fields["book"] = BookGroup(document=self)
         self._fields["prices"] = PriceGroup(document=self)
@@ -134,10 +134,13 @@ class Document(DatabaseObject):
         self._fields["preview"] = PreviewGroup(document=self)
         self._fields["licenses"] = LicensesGroup(document=self)
         self._fields["tags"] = SearchTagsField(document=self)
-        self._fields["admin_tags"] = AdminSearchTagsField(document=self)
+        self._fields["labels"] = LabelsField(document=self)
         self._fields["assets"] = AssetsGroup(self)
         self._fields["processing"] = ProcessingGroup(self)
         self._fields["onix"] = DocumentOnix(self)
+        self._fields["audio"] = AudioGroup(document=self)
+        self._fields["page_count"] = PageCountGroup(document=self)
+        self._fields["products"] = SimpleField(database_object=self, aspect="products.*", field_locator="products", dtype=dict)
 
         self._files = Files(self)
         self._events = ObjectEvents(self)
@@ -150,12 +153,9 @@ class Document(DatabaseObject):
     self_publishing = FieldDescriptor("self_publishing")
     language = FieldDescriptor("language")
     drm = FieldDescriptor("drm")
-    _page_count_technical = FieldDescriptor("_page_count_technical")
-    _page_count_custom = FieldDescriptor("_page_count_custom")
     urls = FieldDescriptor("urls")
     excerpt = FieldDescriptor("excerpt")
     biographical_note = FieldDescriptor("biographical_note")
-    suppliers = FieldDescriptor("suppliers")
 
     book = FieldDescriptor("book")
     prices = FieldDescriptor("prices")
@@ -179,20 +179,17 @@ class Document(DatabaseObject):
     preview = FieldDescriptor("preview")
     licenses = FieldDescriptor("licenses")
     tags = FieldDescriptor("tags")
-    admin_tags = FieldDescriptor("admin_tags")
+    labels = FieldDescriptor("labels")
     assets = FieldDescriptor("assets")
     processing = FieldDescriptor("processing")
     onix = FieldDescriptor("onix")
+    audio = FieldDescriptor("audio")
+    page_count = FieldDescriptor("page_count")
+    internal_note = FieldDescriptor("internal_note")
+    insert_bastard_title = FieldDescriptor("insert_bastard_title")
+    bastardtitle_additional_legal_information = FieldDescriptor("bastardtitle_additional_legal_information")
+    products = FieldDescriptor("products")
 
-    @property
-    def page_count(self):
-        if self._page_count_custom is not None:
-            return self._page_count_custom
-        return self._page_count_technical
-
-    @page_count.setter
-    def page_count(self, value):
-        self._page_count_custom = value
 
     @property
     def files(self):
@@ -219,10 +216,6 @@ class Document(DatabaseObject):
         self._context.gjp.unpublish_document(self.document_id)
         self._fields["status"].invalidate()
 
-    def approve(self):
-        self._context.gjp.final_check_ok(self.document_id)
-        self._fields["status"].invalidate()
-
     def delete(self):
         self._context.gjp.delete_document(self.document_id)
         self._fields["status"].invalidate()
@@ -264,6 +257,18 @@ class DocumentReferences(object):
     def mobi(self):
         return PublicationReferences(self._document, PublicationType.mobi)
 
+    @property
+    def ibooks(self):
+        return PublicationReferences(self._document, PublicationType.ibooks)
+
+    @property
+    def audiobook(self):
+        return PublicationReferences(self._document, PublicationType.audiobook)
+
+    @property
+    def software(self):
+        return PublicationReferences(self._document, PublicationType.software)
+
     @property
     def pod(self):
         return PublicationReferences(self._document, PublicationType.pod)
@@ -373,6 +378,12 @@ class DistributionGroup(FieldGroup):
                                                                 review_count,
                                                                 shop_review_url)
 
+    def set_shoplink(self, shop: str, ean: str, publication_type: str, shoplink: str) -> bool:
+        return ShopLink(self._document.context).set_shoplink(shop, ean, publication_type, shoplink)
+
+    def set_external_id(self, shop: str, ean: str, external_id: str) -> bool:
+        return ExternalID(self._document.context).set_external_id(shop, ean, external_id)
+
 
 class EditionGroup(FieldGroup):
     def __init__(self,
@@ -593,6 +604,12 @@ class BookGroup(FieldGroup):
                                                     dtype=int,
                                                     nullable=True)
         
+        self._fields["colored_pages"] = SimpleField(database_object=document,
+                                            aspect="book.*",
+                                            field_locator="book.colored_pages",
+                                            dtype=str,
+                                            nullable=True)
+        
 
     height = FieldDescriptor('height')
     width = FieldDescriptor('width')
@@ -601,6 +618,7 @@ class BookGroup(FieldGroup):
     weight = FieldDescriptor('weight')
     weight_technical = FieldDescriptor('weight_technical')
     weight_custom = FieldDescriptor('weight_custom')
+    colored_pages = FieldDescriptor('colored_pages')
 
 class URLsGroup(FieldGroup):
     def __init__(self,
@@ -674,3 +692,47 @@ class ProcessingGroup(FieldGroup):
     epub = FieldDescriptor('epub')
     pdf = FieldDescriptor('pdf')
     
+
+class PageCountGroup(FieldGroup):
+    def __init__(self,
+                 document):
+        super(PageCountGroup, self).__init__(document)
+        
+        self._fields["technical"] = SimpleField(database_object=document,
+                                                aspect="ebook.*",
+                                                field_locator="ebook.page_count_technical",
+                                                dtype=int)
+
+        self._fields["custom"] = SimpleField(database_object=document,
+                                             aspect="ebook.*",
+                                             field_locator="ebook.page_count_custom",
+                                             dtype=int,
+                                             nullable=True)
+
+        self._fields["arabic"] = SimpleField(database_object=document,
+                                             aspect="ebook.*",
+                                             field_locator="ebook.page_count_arabic",
+                                             dtype=int,
+                                             nullable=True)
+
+        self._fields["roman"] = SimpleField(database_object=document,
+                                            aspect="ebook.*",
+                                            field_locator="ebook.page_count_roman",
+                                            dtype=str,
+                                            nullable=True)
+        
+    technical = FieldDescriptor('technical')
+    custom = FieldDescriptor('custom')
+    arabic = FieldDescriptor('arabic')
+    roman = FieldDescriptor('roman')
+
+    @property
+    def final(self):
+        if self.custom is not None:
+            return self.custom
+        return self.technical
+    
+    @final.setter
+    def final(self, value):
+        self.custom = value
+
