@@ -71,20 +71,6 @@ class Order(DatabaseObject):
                                                   nullable=True,
                                                   kind=FieldKind.readonly)
 
-        self._fields['user_external_id'] = SimpleField(database_object=self,
-                                                       aspect='*',
-                                                       field_locator='user_external_id',
-                                                       dtype=str,
-                                                       nullable=True,
-                                                       kind=FieldKind.readonly)
-
-        self._fields['payment_external_id'] = SimpleField(database_object=self,
-                                                       aspect='*',
-                                                       field_locator='payment_external_id',
-                                                       dtype=str,
-                                                       nullable=True,
-                                                       kind=FieldKind.readonly)
-
         self._fields['shipping_address'] = AddressGroup(order=self,
                                                         address_locator='shipping_address')
 
@@ -108,8 +94,6 @@ class Order(DatabaseObject):
     distribution_channel = FieldDescriptor('distribution_channel')
     channel_id = FieldDescriptor('channel_id')
     external_id = FieldDescriptor('external_id')
-    user_external_id = FieldDescriptor('user_external_id')
-    payment_external_id = FieldDescriptor('payment_external_id')
     shipping_address = FieldDescriptor('shipping_address')
     invoice_address = FieldDescriptor('invoice_address')
     items = FieldDescriptor('items')
@@ -253,9 +237,7 @@ class AddressGroup(FieldGroup):
                 try:
                     field.update(gjp)
                 except Exception as e:
-                    raise RuntimeError("Update failed for field: '{0}'\n{1}"
-                                       .format(name,
-                                               ''.join(traceback.format_exception_only(type(e), e))))
+                    raise RuntimeError("Update failed for field: '{0}'".format(name)) from e
 
     def __str__(self):
         return '{firstname} {lastname}; {street}; {line2}; {line3}; {city} {state} {zip}; {country}'.format(
@@ -532,10 +514,8 @@ class PersonGroup(FieldGroup):
                 try:
                     field.update(gjp)
                 except Exception as e:
-                    raise RuntimeError("Update failed for field: '{0}'\n{1}"
-                                       .format(name,
-                                               ''.join(traceback.format_exception_only(type(e), e))))
-        
+                    raise RuntimeError("Update failed for field: '{0}'".format(name)) from e
+
 
 def Person(id = None,
            first_name = None,
